Attribute VB_Name = "Main"
Option Explicit

Public Sub CC_AddColumnBreak()
  Dim selPar As Word.Range: Set selPar = ActiveDocument.ActiveWindow.Selection.Paragraphs.First.Range
  Call selPar.Collapse(wdCollapseStart)
  Call selPar.InsertBreak(wdColumnBreak)
End Sub

Public Sub CC_NonBreakPar()
  ActiveDocument.ActiveWindow.Selection.ParagraphFormat.KeepTogether = Not ActiveDocument.ActiveWindow.Selection.ParagraphFormat.KeepTogether
End Sub

Public Sub CC_WithNext()
  Selection.ParagraphFormat.KeepWithNext = Not Selection.ParagraphFormat.KeepWithNext
End Sub

Public Sub CC_FieldLowerer()
  Dim tmpRange As Word.Range: Set tmpRange = ActiveDocument.ActiveWindow.Selection.Range.Duplicate
  tmpRange.Start = ActiveDocument.ActiveWindow.Selection.Words.First.Start
  tmpRange.End = ActiveDocument.ActiveWindow.Selection.Words.Last.End
  
  On Error GoTo EXT_SUB
  Dim execField As Word.Field
  For Each execField In tmpRange.Fields
    Call LowerReference(execField)
  Next execField
EXT_SUB:
End Sub

Public Sub CC_MakeUniqueBM()
  Dim contextMap As Scripting.Dictionary: Set contextMap = RecreateBookmarks(ActiveDocument)
  Call FixHyperlinksAfterBMReplace(ActiveDocument, contextMap)
  Call UserInteraction.ShowMessage(IM_UNIQUE_BOOKMARKS_OK)
End Sub

Public Sub CC_ConvertAllShapesToText()
'    
  Dim theDoc As Word.Document: Set theDoc = ActiveDocument
  Dim tRange As Word.Range: Set tRange = theDoc.Range
  
  Dim nShape&: nShape = 1
  Dim theText$
  
  Dim aShape As Word.Shape
  Do While nShape <= theDoc.Shapes.Count  '        
    Set aShape = theDoc.Shapes(nShape)
    If Not aShape.Anchor.InRange(tRange) Then _
      GoTo NEXT_SHAPE
    If aShape.Type <> msoTextBox Then _
      GoTo NEXT_SHAPE
    
    theText = aShape.TextFrame.TextRange.Text
    
    aShape.TextFrame.TextRange.Copy
    tRange.Paragraphs.Add
    Call tRange.Paragraphs.Last.Range.Paste
    
NEXT_SHAPE:
    nShape = nShape + 1
  Loop
  
  Call UserInteraction.ShowMessage(IM_SHAPES_TO_TEXT_OK)
End Sub

Public Sub CC_Convert1252_1251()
'   
  Dim theDoc As Word.Document: Set theDoc = ActiveDocument
  theDoc.ActiveWindow.View.ReadingLayout = False
  
  Dim inputR As Word.Range: Set inputR = theDoc.Application.Selection.Range
  
  Dim wordUI As New API_WordWrapper: Call wordUI.SetDocument(theDoc)
  Call wordUI.PauseUI
  
  Call CSE_ProgressBar.Init("  1252 -> 1251", sHeader:=" ...", maxVal:=inputR.Characters.Count)
  Call CSE_ProgressBar.ShowModeless

  Dim aChr As Word.Range: Set aChr = inputR.Characters.First
  Do While aChr.End < inputR.End
    Dim ChrW&: ChrW = VBA.AscW(aChr)
    If ChrW < 256 And ChrW > 127 Then _
      aChr.Text = VBA.Chr(ChrW)
    Set aChr = aChr.Next(wdCharacter, 1)
    Call CSE_ProgressBar.IncrementA
  Loop
  
  Call Unload(CSE_ProgressBar)
  Call wordUI.ResumeUI
  Call UserInteraction.ShowMessage(IM_CONVERT_CODEPAGE_OK)
End Sub

Public Sub CC_ParFix()
' Modify spacing to make last line optimal length
  Dim selRange As Word.Range: Set selRange = ActiveDocument.ActiveWindow.Selection.Range.Duplicate
  Dim aPar As Word.Paragraph
  For Each aPar In selRange.Paragraphs
    Dim tRange As Word.Range:  Set tRange = aPar.Range
    If tRange.Tables.Count > 0 Then tRange.End = tRange.End - 1
    Call tRange.Select
    Call TryFixingParagraph(tRange)
  Next aPar
End Sub

Public Sub CC_ExportComments()
  ' Export comments in new document
  Dim iSource As Word.Document: Set iSource = ActiveDocument
  Dim iDestination As Word.Document: Set iDestination = Word.Application.Documents.Add
  
  Dim targetRange As Word.Range: Set targetRange = iDestination.Range
  Dim aComment As Word.Comment
  Dim nCount&: nCount = 0
  Dim nPage&
  For Each aComment In iSource.Comments
    nCount = nCount + 1
    Set targetRange = targetRange.Paragraphs.Add.Range
    nPage = aComment.Reference.Information(wdActiveEndPageNumber)
    targetRange.Text = Fmt(" {1},  {2},  {3}:" & vbNewLine & "{4}" & vbNewLine, nCount, nPage, aComment.Author, aComment.Range.Text)
  Next aComment
  
  Call UserInteraction.ShowMessage(IM_COMMENT_EXPORT_OK, nCount)
End Sub

Public Sub CC_LineUp()
  Dim tRange As Word.Range: Set tRange = ActiveDocument.ActiveWindow.Selection.Paragraphs(1).Range
  Dim iSpace As Double: iSpace = tRange.ParagraphFormat.SpaceBefore + tRange.ParagraphFormat.LineSpacing
  tRange.ParagraphFormat.SpaceBefore = iSpace
End Sub

Public Sub CC_LineUpA()
  Dim tRange As Word.Range: Set tRange = ActiveDocument.ActiveWindow.Selection.Paragraphs(1).Range
  Dim iSpace As Double:  iSpace = tRange.ParagraphFormat.SpaceBefore - tRange.ParagraphFormat.LineSpacing
  tRange.ParagraphFormat.SpaceBefore = IIf(iSpace >= 0, iSpace, 0)
End Sub

Public Sub CC_LineDown()
  Dim tRange As Word.Range: Set tRange = ActiveDocument.ActiveWindow.Selection.Paragraphs(1).Range
  Dim iSpace As Double: iSpace = tRange.ParagraphFormat.SpaceAfter + tRange.ParagraphFormat.LineSpacing
  tRange.ParagraphFormat.SpaceAfter = iSpace
End Sub

Public Sub CC_LineDownA()
  Dim tRange As Word.Range: Set tRange = ActiveDocument.ActiveWindow.Selection.Paragraphs(1).Range
  Dim iSpace As Double: iSpace = tRange.ParagraphFormat.SpaceAfter - tRange.ParagraphFormat.LineSpacing
  tRange.ParagraphFormat.SpaceAfter = IIf(iSpace >= 0, iSpace, 0)
End Sub

Public Sub CC_CheckLinks()
'  
  Dim thisDoc As Word.Document: Set thisDoc = ActiveDocument
  Dim linkCount&: linkCount = thisDoc.Hyperlinks.Count
  If thisDoc.Hyperlinks.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_HYPERLINKS_MISSING)
    Exit Sub
  End If
  
  Dim result As LMU_LinksInfo: result = FixHyperlinksIn(ActiveDocument.Range)
  Call UserInteraction.ShowMessage(IM_HYPERLINKS_FIX_OK, result.lnkCount_, result.errCount_, result.modCount_)
End Sub

Public Sub CC_AddQuote()
  ActiveDocument.ActiveWindow.Selection.Range.Text = Chr(171) & ActiveDocument.ActiveWindow.Selection.Text & Chr(187)
End Sub

Public Sub CC_EditCut()
'     
  Dim aRange As Word.Range: Set aRange = ActiveDocument.ActiveWindow.Selection.Range
  aRange.Start = aRange.Paragraphs.First.Range.Start
  aRange.End = aRange.Paragraphs.Last.Range.End
  
  Call aRange.Cut
  Call ActiveDocument.Paragraphs.Add
  Call ActiveDocument.Paragraphs.Last.Range.Paste
End Sub

Public Sub CC_EditCopy()
'     
  Dim aRange As Word.Range: Set aRange = ActiveDocument.ActiveWindow.Selection.Range
  aRange.Start = aRange.Paragraphs.First.Range.Start
  aRange.End = aRange.Paragraphs.Last.Range.End
  
  Call aRange.Copy
  
  Call ActiveDocument.Paragraphs.Add
  Call ActiveDocument.Paragraphs.Last.Range.Paste
  
  aRange.HighlightColorIndex = wdYellow
End Sub

Public Sub CC_CreateHLink()
  Dim target As Word.Range: Set target = ActiveDocument.ActiveWindow.Selection.Range
  
  Dim bookmarkID$
  Dim aBM As Word.Bookmark
  For Each aBM In target.Bookmarks
    If aBM.Range.End = target.End And aBM.Range.Start = target.Start Then bookmarkID = aBM.Name
  Next aBM
  
  If bookmarkID = vbNullString Then
    On Error GoTo ERROR_MSG
    bookmarkID = "AUTO_" & Format(Date, "yyyymmdd") & Replace(Time, ":", "")
    Call target.Bookmarks.Add(bookmarkID)
  End If
  
  Call AddToClipboard(target.Text)
  DoEvents
  Call AddToClipboard(ActiveDocument.FullName & "#" & bookmarkID)
  Exit Sub
  
ERROR_MSG:
  Call UserInteraction.ShowMessage(EM_HYPERLINK_CREATION_FAIL)
End Sub

Public Sub CC_RemoveExternalHyperlinks()
  Dim target As Word.Range: Set target = ActiveDocument.ActiveWindow.Selection.Range.Duplicate
  Dim aLink As Hyperlink
  Dim nLink&: nLink = 1
  Dim delCount&: delCount = 0
  Do While target.Hyperlinks.Count >= nLink
    Set aLink = target.Hyperlinks.Item(nLink)
    If aLink.Address <> vbNullString Then
      Call aLink.Range.Font.Reset
      delCount = delCount + 1
      Call aLink.Delete
    Else
      nLink = nLink + 1
    End If
  Loop
  Call Word.Application.ScreenRefresh
  Call UserInteraction.ShowMessage(IM_REMOVE_HYPERLINKS_OK, delCount)
End Sub

Public Sub CC_RemoveAllHyperlinks()
  Dim target As Word.Range: Set target = ActiveDocument.ActiveWindow.Selection.Range.Duplicate
  Dim aLink As Hyperlink
  Dim delCount&: delCount = target.Hyperlinks.Count
  Do While target.Hyperlinks.Count > 0
    Set aLink = target.Hyperlinks.Item(1)
    Call aLink.Range.Font.Reset
    Call aLink.Delete
  Loop
  Call Word.Application.ScreenRefresh
  Call UserInteraction.ShowMessage(IM_REMOVE_HYPERLINKS_OK, delCount)
End Sub

Public Sub CC_RemoveImages()
  Dim nDeleted&: nDeleted = RemoveImages(ActiveDocument.ActiveWindow.Selection.Range.Duplicate)
  Call Word.Application.ScreenRefresh
  Call UserInteraction.ShowMessage(IM_REMOVE_IMAGES_OK, nDeleted)
End Sub

Public Sub CC_RemoveBookmarks()
  Dim target As Word.Range: Set target = ActiveDocument.ActiveWindow.Selection.Range.Duplicate
  Dim aBM As Word.Bookmark
  Dim delCount&: delCount = target.Bookmarks.Count
  Do While target.Bookmarks.Count > 0
    Set aBM = target.Bookmarks.Item(1)
    Call aBM.Delete
  Loop
  Call Word.Application.ScreenRefresh
  Call UserInteraction.ShowMessage(IM_REMOVE_BOOKMARKS_OK, delCount)
End Sub

Public Sub CC_RemoveAccents()
  Dim target As Word.Range: Set target = ActiveDocument.ActiveWindow.Selection.Range.Duplicate
  If target.Start = target.End Then _
    Set target = ActiveDocument.Range
  
  Dim nCount&: nCount = RemoveAccentsIn(target)
  Call Word.Application.ScreenRefresh
  Call UserInteraction.ShowMessage(IM_REMOVE_ACCENTS_OK, nCount)
End Sub

Public Sub CC_MasterText()
  Dim sPath$: sPath = Word.ActiveDocument.FullName
  If Word.ActiveDocument.Path = vbNullString Then
    Call UserInteraction.ShowMessage(EM_DOCUMENT_NOT_SAVED)
    Exit Sub
  End If
  
  Dim xlWrap As New API_XLWrapper
  Dim aDoc As Excel.Workbook: Set aDoc = xlWrap.OpenDocument(SERVER_MASTER_TEXT)
  If aDoc Is Nothing Then _
    Exit Sub

  Call aDoc.Activate
  aDoc.Names(CELL_MASTER_PATH).RefersToRange = sPath$
End Sub

Public Sub CC_UpdateConcept()
  If Not UpdateConceptPowershell Then
    Call UserInteraction.ShowMessage(EM_CORE_UPDATE_FAILED)
    Exit Sub
  End If
  Call UserInteraction.ShowMessage(IM_CONCEPT_UPDATE_OK)
End Sub

Public Sub CC_Checkup()
  Call ExecuteAllRules(ActiveDocument, bApplyFix:=False)
End Sub

Public Sub CC_FixErrors()
  Call ExecuteAllRules(ActiveDocument, bApplyFix:=True)
End Sub

Public Sub CC_MasterConfig()
  Call OpenMasterConfig
End Sub

Public Sub CC_IgnoreSelected()
  '       
  Dim target As Word.Range:  Set target = ActiveDocument.Application.Selection.Range
  If target.Start = target.End Then
    Call UserInteraction.ShowMessage(EM_SELECTION_EMPTY)
    Exit Sub
  End If
  
  Dim iEditor As New API_WordEditGuard: Call iEditor.InitDoc(target.Document)
  Dim sBookmark$: sBookmark = iEditor.BookmarkIgnored(target)
  Call UserInteraction.ShowMessage(IM_BOOKMARK_IGNORED, sBookmark)
End Sub

Public Sub CC_Help()
  Dim wordWrap As New API_WordWrapper
  Dim helpDoc As Word.Document: Set helpDoc = wordWrap.OpenDocument(SERVER_PATH_HELP, bReadOnly:=True)
  If helpDoc Is Nothing Then _
    Exit Sub

  With helpDoc
    .ActiveWindow.View.ReadingLayout = False
    Call .Application.Activate
  End With
End Sub

' ============
Private Function ExecuteAllRules(iDoc As Word.Document, Optional bApplyFix As Boolean = True)
  Dim iConfig As API_Config: Set iConfig = LoadEditConfig
  Dim iProcessor As New RulesProcessor: Call iProcessor.Init(iDoc, iConfig)
  Dim iGuard As New API_WordEditGuard: Call iGuard.InitDoc(iDoc)
  
  Call iGuard.BeginEdit(iConfig.GetValue("TrackRevisions"))
  
  Call CSE_ProgressBar.Init("  ", sHeader:=" , ...", maxVal:=iProcessor.RuleCount, canInterrupt:=True)
  Call CSE_ProgressBar.ShowModeless
  
  Call iProcessor.RunAll(CollectionToDictionary(iConfig.GetValue("ExcludeRules")), bApplyFix)
  
  Dim nMilliseconds&: nMilliseconds = Int(CSE_ProgressBar.Time)
  CSE_ProgressBar.Description = " "
  Call iGuard.EndEdit
  
  Call Unload(CSE_ProgressBar)
  Call UserInteraction.ShowMessage(IM_RULES_COMPLETE, nMilliseconds, iProcessor.errors_, iProcessor.fixes_)
End Function
