Attribute VB_Name = "RulesAccess"
' Grant access to document processing rules
Option Explicit

Public Function AccessProcessor() As RulesProcessor
  Static s_Processor As RulesProcessor
  If s_Processor Is Nothing Then _
    Set s_Processor = New RulesProcessor
  Set AccessProcessor = s_Processor
End Function

Public Function InitProcessor(sDocument$, sConfigJSON$) As Boolean
  On Error GoTo RETURN_FALSE
  Dim iWrap As New API_WordWrapper
  If iWrap.OpenDocument(sDocument) Is Nothing Then _
    GoTo RETURN_FALSE
  
  Dim iConfig As New API_Config: Call iConfig.LoadFromJSON(sConfigJSON)
  Call AccessProcessor.Init(iWrap.Document, iConfig)
  InitProcessor = True
  Exit Function
  
RETURN_FALSE:
  InitProcessor = False
End Function

Public Function RunRule(sRuleID$, Optional bApplyFix As Boolean = True) As Long()
  Dim iResult() As Long
  ReDim iResult(0 To 2)
  
  Dim iProcessor As RulesProcessor: Set iProcessor = AccessProcessor
  
  On Error GoTo RETURN_ERROR
  Call iProcessor.ResetCounters
  Call iProcessor.RunRule(sRuleID, bApplyFix)
  On Error GoTo 0
  
  iResult(0) = 0
  iResult(1) = iProcessor.errors_
  iResult(2) = iProcessor.fixes_
  RunRule = iResult
  Exit Function
  
RETURN_ERROR:
  iResult(0) = Err.Number
  RunRule = iResult
End Function

Public Function FinalizeRulesProcessing()
  Call AccessProcessor.FinalizeProcessing
End Function

Public Function EmptyEditConfig() As API_Config
  Set EmptyEditConfig = New API_Config
  With EmptyEditConfig
    Call .SetValue("HighlightErrors", False)
    Call .SetValue("CommentUnresolved", True)
    Call .SetValue("TrackRevisions", False)
    Call .SetValue("ExcludeRules", New Collection)
    Call .SetValue("BannedWords", New Collection)
  End With
End Function

Public Function DefaultEditConfig() As API_Config
  Set DefaultEditConfig = New API_Config
  With DefaultEditConfig
    Call .SetValue("HighlightErrors", False)
    Call .SetValue("CommentUnresolved", True)
    Call .SetValue("TrackRevisions", True)
    Call .SetValue("ExcludeRules", New Collection)
    Call .SetValue("BannedWords", LoadBannedWords)
  End With
End Function

Public Function LoadEditConfig() As API_Config
  Dim iConfig As API_Config
  Dim fso As New Scripting.FileSystemObject
  Dim sFile$: sFile = VBA.Environ$("USERPROFILE") & "\" & CONCEPT_LOCAL_HOME & "\" & CONCEPT_MASTER_CONFIG
  If fso.FileExists(sFile) Then
    Set iConfig = New API_Config
    Call iConfig.LoadFromFile(sFile)
  Else
    Set iConfig = DefaultEditConfig
    Call iConfig.RemoveKey("BannedWords")
    Call iConfig.SaveToFile(sFile)
  End If
  Call iConfig.SetValue("BannedWords", LoadBannedWords)
  Set LoadEditConfig = iConfig
End Function

Public Function OpenMasterConfig()
  Dim fso As New Scripting.FileSystemObject
  Dim sFile$: sFile = VBA.Environ$("USERPROFILE") & "\" & CONCEPT_LOCAL_HOME & "\" & CONCEPT_MASTER_CONFIG
  If Not fso.FileExists(sFile) Then
    Dim iConfig As API_Config: Set iConfig = DefaultEditConfig
    Call iConfig.RemoveKey("BannedWords")
    Call iConfig.SaveToFile(sFile)
  End If
  Dim oShell As New Shell32.Shell
  Call oShell.Open(sFile)
End Function

' =======
Private Function LoadBannedWords() As Collection
  Dim result As New Collection
  Dim sLocal$: sLocal = VBA.Environ$("USERPROFILE") & "\" & CONCEPT_LOCAL_HOME & "\models\" & CONCEPT_BANNED_WORDS
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(sLocal) Then
    If Not TryUpdateBannedWords() Then
      Call UserInteraction.ShowMessage(EM_MISSING_FILE, sLocal)
      GoTo RETURN_EMPTY
    End If
  End If

  Dim adoStream As New ADODB.Stream
  adoStream.Charset = "utf-8"
  Call adoStream.Open
  Call adoStream.LoadFromFile(sLocal)
  Dim sLines() As String: sLines = VBA.Split(adoStream.ReadText, vbCrLf)
  Call adoStream.Close
  
  Dim nLine&
  For nLine = LBound(sLines, 1) To UBound(sLines, 1) Step 1
    Dim sText$: sText = sLines(nLine)
    If sText <> vbNullString Then _
      Call result.Add(sText)
  Next nLine
  
RETURN_EMPTY:
  Set LoadBannedWords = result
End Function

Private Function TryUpdateBannedWords() As Boolean
  Dim sServer$: sServer = CONCEPT_SERVER_HOME & CONCEPT_BANNED_WORDS
  Dim sLocal$: sLocal = VBA.Environ$("USERPROFILE") & "\" & CONCEPT_LOCAL_HOME & "\models\" & CONCEPT_BANNED_WORDS
  TryUpdateBannedWords = CopyFileOrFolder(sSource:=sServer, sDestination:=sLocal)
End Function
