VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "RulesProcessor"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Const SYMBOL_QUOTE$ = """"
Private Const SYMBOL_LETTER$ = "a-zA-Z--߸"
Private Const SYMBOL_DIGIT$ = "0-9"

Private Const MASK_LETTER$ = "[" & SYMBOL_LETTER & "]"
Private Const MASK_DIGIT$ = "[" & SYMBOL_DIGIT & "]"
Private Const MASK_ALPHANUM = "[" & SYMBOL_LETTER & SYMBOL_DIGIT & "]"
Private Const MASK_ROMAN_DIGIT$ = "[ivxlcdmIVXLCDM]"

Private Const MASK_QUOTE$ = "[" & SYMBOL_QUOTE & "]"

Private Const NEW_PARAGRAPH = -1

'      
Private Const AR1019_LINES_PER_PAR = 10

Private Const TEXT_SPELLING_STATS = "{1}" & vbNewLine & " : {2}" & vbNewLine & " : {3}"

Private editor_ As DocumentEditor
Private config_ As API_Config
Private applyFix_ As Boolean

Public errors_ As Long
Public fixes_ As Long

Private Sub Class_Initialize()
  Call ResetCounters
End Sub

Public Property Get RuleCount() As Long
  RuleCount = 30
End Property

Public Function Init(iDoc As Word.Document, iConfig As API_Config)
  Set editor_ = New DocumentEditor
  Call editor_.Init(iDoc, iConfig)
  Set config_ = iConfig
  Call ResetCounters
End Function

Public Function ResetCounters()
  errors_ = 0
  fixes_ = 0
End Function

Public Function RunAll(iExclude As Scripting.Dictionary, bApplyFix As Boolean)
  applyFix_ = bApplyFix
  Call RunRulesInternal(iExclude)
  Call FinalizeProcessing
End Function

Public Function RunRule(sRuleID$, bApplyFix As Boolean)
  Dim sFunction$: sFunction = "Rule" & VBA.Right(sRuleID, VBA.Len(sRuleID) - 2)
  applyFix_ = bApplyFix
  Call CallByName(Me, sFunction, VbMethod)
End Function

Public Function FinalizeProcessing()
  Call editor_.FinalizeComments
End Function

Public Function Rule1001()
  Const RULE_DESCRIPTION$ = "AR1001:    /  "
  
  Dim rFind As Word.Range: Set rFind = editor_.SetupFindLiteral("^p ")
  Do While rFind.Find.Execute
    Call rFind.MoveEndWhile(CSet:=" " & Chr(160), Count:=wdForward)
    
    If Not applyFix_ Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION$)
    Else
      rFind.Start = rFind.Start + 1
      rFind.Text = vbNullString
    End If
    
    Call rFind.Collapse(wdCollapseEnd)
    Call Increment(applyFix_)
  Loop
  
  Set rFind = editor_.SetupFindLiteral(" ^p")
  Do While rFind.Find.Execute
    Call rFind.MoveStartWhile(CSet:=" " & Chr(160), Count:=wdBackward)
    If Not applyFix_ Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION$)
    Else
      rFind.End = rFind.End - 1
      rFind.Text = vbNullString
    End If
    
    Call rFind.Collapse(wdCollapseEnd)
    Call Increment(applyFix_)
  Loop
  
  Set rFind = editor_.SetupFindLiteral("^t^p")
  Do While rFind.Find.Execute
    Call rFind.MoveStartWhile(CSet:=" " & Chr(160) & vbTab, Count:=wdBackward)
    If Not applyFix_ Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION$)
    Else
      rFind.End = rFind.End - 1
      rFind.Text = vbNullString
    End If
    
    Call rFind.Collapse(wdCollapseEnd)
    Call Increment(applyFix_)
  Loop
End Function

Public Function Rule1002()
  Const RULE_DESCRIPTION$ = "AR1002:   "

  Dim rFind As Word.Range: Set rFind = editor_.SetupFindLiteral("  ")
  Do While rFind.Find.Execute
    Call rFind.MoveEndWhile(CSet:=" ", Count:=wdForward)
    If Not applyFix_ Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION)
    Else
      rFind.Start = rFind.Start + 1
      rFind.Text = vbNullString
    End If
    
    Call rFind.Collapse(wdCollapseEnd)
    Call Increment(applyFix_)
  Loop
End Function

Public Function Rule1003()
  Const RULE_DESCRIPTION$ = "AR1003:   "
  
  Dim rFind As Word.Range: Set rFind = editor_.SetupFindLiteral("^p^p")
  Do While rFind.Find.Execute
    If Not applyFix_ Or rFind.End - rFind.Start <> 2 Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION)
      Call IncrementErrors
    Else
      Call rFind.MoveEndWhile(CSet:=vbNewLine, Count:=wdForward)
      rFind.End = rFind.End - 1 ' Note: if Start is adjusted, then will have problems with end of document
      rFind.Text = vbNullString
      Call Increment(doBoth:=True)
    End If
    
    Call rFind.Collapse(wdCollapseEnd)
  Loop
End Function

Public Function Rule1004()
  Const RULE_DESCRIPTION$ = "AR1004:  -"
  
  Dim result&: result = 0
  result = result + editor_.ReplaceText("+/-", "", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceText("+ / -", "", applyFix_, RULE_DESCRIPTION)
  Call Increment(applyFix_, result)
End Function

Public Function Rule1005()
  Const RULE_DESCRIPTION$ = "AR1005:   "

  Call Increment(applyFix_, editor_.ReplaceCascade("[ ^0160][-^0150][ ^0160]", "[-^0150]", Chr(151), applyFix_, RULE_DESCRIPTION, bWildcard:=True))
End Function

Public Function Rule1006()
  Const RULE_DESCRIPTION$ = "AR1006:  "

  Dim result&: result = 0
  result = result + editor_.ReplaceCascade(MASK_LETTER & "^0151" & MASK_LETTER, "^0151", "-", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade(MASK_LETTER & "^0150" & MASK_LETTER, "^0150", "-", applyFix_, RULE_DESCRIPTION)
  Call Increment(applyFix_, result)
End Function

Public Function Rule1007()
  Const RULE_DESCRIPTION$ = "AR1007:   "
  
  Dim result&: result = 0
  result = result + editor_.ReplaceCascade("<N^0032" & MASK_DIGIT, "N^0032", "^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<N^0160" & MASK_DIGIT, "N", "", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<N" & MASK_DIGIT, "N", "^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("" & MASK_DIGIT, "", "^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("^0032" & MASK_DIGIT, "^0032", "^0160", applyFix_, RULE_DESCRIPTION)
  Call Increment(applyFix_, result)
End Function

Public Function Rule1008()
  Const RULE_DESCRIPTION$ = "AR1008:   "
  
  Dim result&: result = 0
  result = result + editor_.ReplaceWC("[^0032^0160][^0150^0151][!^0032^0160^0013]", "", False, RULE_DESCRIPTION)
  result = result + editor_.ReplaceWC("[!^0032^0160][^0150^0151][^0032^0160^0013]", "", False, RULE_DESCRIPTION)
  result = result + editor_.ReplaceWC(MASK_DIGIT & "[^0032^0160][^0045^0150^0151][^0032^0160]" & MASK_DIGIT, "", False, RULE_DESCRIPTION)
  Call IncrementErrors(result)
End Function

Public Function Rule1009()
  Const RULE_DESCRIPTION$ = "AR1009: "
  
  Dim rFind As Word.Range: Set rFind = editor_.SetupFindLiteral("...")
  Do While rFind.Find.Execute
    Call rFind.MoveEndWhile(CSet:=".", Count:=wdForward)
    If Not applyFix_ Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION)
    Else
      rFind.Text = Chr(133)
    End If
    
    Call rFind.Collapse(wdCollapseEnd)
    Call Increment(applyFix_)
  Loop
  
  If applyFix_ Then
    Dim nFixes&: nFixes = 1
    Do Until nFixes = 0
      nFixes = editor_.ReplaceText(".^0133", "^0133", applyFix_, RULE_DESCRIPTION)
      nFixes = nFixes + editor_.ReplaceText("^0133.", "^0133", applyFix_, RULE_DESCRIPTION)
      nFixes = nFixes + editor_.ReplaceText("^0133^0133", "^0133", applyFix_, RULE_DESCRIPTION)
      Call Increment(applyFix_, nFixes)
    Loop
  End If
End Function

Public Function Rule1010()
  Const RULE_DESCRIPTION$ = "AR1010:     "
  
  Dim rFind As Word.Range
  Dim aStyle As Variant
  For Each aStyle In editor_.Document.Styles
    If Not aStyle.NameLocal Like "[]*" Then _
      GoTo NEXT_STYLE
      
    Set rFind = editor_.SetupFindLiteral(".^p")
    rFind.Find.Style = aStyle.NameLocal
    Do While rFind.Find.Execute
      If rFind.Previous(wdWord, 1) Is Nothing Then _
        GoTo FIND_NEXT
    
      Dim lastword$: lastword = rFind.Previous(wdWord, 1).Text
      If Not lastword = "" And Not lastword = "" Then
        If Not applyFix_ Then
          Call editor_.MarkError(rFind.Words(1), RULE_DESCRIPTION)
        Else
          Call rFind.Words(1).Delete
        End If
        Call Increment(applyFix_)
      End If
FIND_NEXT:
      Call rFind.Collapse(wdCollapseEnd)
    Loop
NEXT_STYLE:
  Next aStyle
End Function

Public Function Rule1011()
  Const RULE_DESCRIPTION$ = "AR1011:  . / ."

  Dim result&: result = 0
  result = result + ReplaceAliasDig(".", RULE_DESCRIPTION)
  result = result + ReplaceAliasDig(".", RULE_DESCRIPTION)
  Call Increment(applyFix_, result)
End Function

Public Function Rule1012()
  Const RULE_DESCRIPTION$ = "AR1012:   .  ."
  
  Dim result&: result = 0
  result = result + editor_.ReplaceCascade("<[]>. [0-9]", " ", Chr(160), applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[]>. [0-9]", " ", Chr(160), applyFix_, RULE_DESCRIPTION)
  Call Increment(applyFix_, result)
End Function

Public Function Rule1013()
  Const RULE_DESCRIPTION$ = "AR1013:    "
  
  Dim rFind As Word.Range: Set rFind = editor_.SetupFindLiteral("=")
  Do While rFind.Find.Execute(Replace:=False)
    If rFind.OMaths.Count = 0 Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION)
      Call IncrementErrors
    End If
    Call rFind.Collapse(wdCollapseEnd)
  Loop
End Function

Public Function Rule1014()
  Const RULE_DESCRIPTION$ = "AR1014:   "
  
  Dim result&: result = 0
  Dim sReplacement$: sReplacement = "^0160^0160"

  result = result + editor_.ReplaceWC(" [ ^0160]", sReplacement, applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceWC("[ ^0160] ", sReplacement, applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceWC("[ ^0160][ ^0160][""][""]", sReplacement, applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceWC("[ ^0160][ ^0160][""][""]", sReplacement, applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceWC("[ ^0160]" & MASK_QUOTE & "[ ^0160]" _
              & MASK_QUOTE & "" & MASK_QUOTE & MASK_QUOTE, sReplacement, applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceWC("[ ^0160]" & MASK_QUOTE & "[ ^0160]" _
              & MASK_QUOTE & "" & MASK_QUOTE & MASK_QUOTE, sReplacement, applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceWC("[ ^0160][ ^0160]һ", sReplacement, applyFix_, RULE_DESCRIPTION)
  
  Call Increment(applyFix_, result)
End Function

Public Function Rule1016()
  Const RULE_DESCRIPTION$ = "AR1016: . . "
  
' TODO: Refactor
  Dim newText$, leftC$, rightC$
  Dim leftWord As Word.Range, rightWord As Word.Range
  Dim leftFlag As Boolean, rightFlag As Boolean
  
  Dim rFind As Word.Range
  Set rFind = editor_.SetupFindWildcard("[-].[ ^0160][-].")
  Do While rFind.Find.Execute
    leftC = rFind.Previous(wdCharacter, 1).Text
    rightC = rFind.Next(wdCharacter, 1).Text
    
    If leftC Like "[--0-9]" Then _
      GoTo FIND_NEXT
          
    '     
    Set leftWord = rFind.Previous(wdWord, 1)
    Set rightWord = rFind.Next(wdWord, 1)
    Call rightWord.MoveEndWhile(CSet:=" ^t", Count:=wdBackward) '    
    
    leftFlag = False
    rightFlag = False
    If rightC Like "[-]" Then
      If Not rFind.Next(wdCharacter, 2) Like "[-]" Then _
        GoTo FIND_NEXT
      
      Set rightWord = rFind.Next(wdCharacter, 3)
      Do While rightWord.Text Like "[-]"
         Set rightWord = rightWord.Next(wdCharacter, 1)  '     
      Loop
      
      rightFlag = True
      rightWord.Start = rFind.End + 2
      rightWord.End = rightWord.End - 1
    Else
      If Not leftC = Chr(160) And Not rightC = Chr(160) Then
        leftFlag = RangeIsSurname(leftWord)
        If rightC Like "[;,:?!.]" Then
          rightFlag = False
        Else
          rightFlag = RangeIsSurname(rightWord)
        End If
      Else
        If leftC = Chr(160) Then
          Set leftWord = rFind.Previous(wdCharacter, 2)
          Do While leftWord.Text Like "[-]"
            Set leftWord = leftWord.Previous(wdCharacter, 1)   '     
          Loop
          
          leftFlag = leftWord.Start < rFind.Start - 2 And leftWord.Text Like "[-]"
          leftWord.End = rFind.Start - 1
        End If
        If rightC = Chr(160) And rFind.Next(wdCharacter, 2) Like "[-]" And _
            rFind.Next(wdCharacter, 3) Like "[-]" Then
          Set rightWord = rFind.Next(wdCharacter, 4)
          Do While rightWord.Text Like "[-]"
             Set rightWord = rightWord.Next(wdCharacter, 1)  '     
          Loop
          
          '    
          If rFind.Characters(3) = Chr(160) Then _
            GoTo FIND_NEXT
          
          rightFlag = True
          rightWord.Start = rFind.End + 2
          rightWord.End = rightWord.End - 1
        End If
      End If
    End If
    
    If Not leftFlag And Not rightFlag Then _
      GoTo FIND_NEXT
    
    If Not applyFix_ Or (leftFlag And rightFlag) Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION)
      Call IncrementErrors
      GoTo FIND_NEXT
    End If
    
    newText = rFind.Text
    newText = Left(newText, 2) & Chr(160) & Right(newText, 2) & Chr(160)
    If leftFlag Then  '  ..
      rFind.Start = leftWord.Start
      
      newText = newText & Trim(leftWord.Text)
      rFind.Text = newText
    Else  ' .. 
      rFind.Start = rFind.Start
      rFind.End = rightWord.End
      
      newText = newText & Trim(rightWord.Text)
    End If
    
    If rFind.Fields.Count > 0 Then _
      GoTo FIND_NEXT
    
    rFind.Text = newText
    Call Increment(doBoth:=True)

FIND_NEXT:
    Call rFind.Collapse(wdCollapseEnd)
  Loop
  
  Set rFind = editor_.SetupFindWildcard("[-].[-].")
  Do While rFind.Find.Execute
    '     
    leftC = rFind.Previous(wdCharacter, 1).Text
    rightC = rFind.Next(wdCharacter, 1).Text
    
    If leftC Like "[--0-9]" Then _
      GoTo FIND_NEXT2
          
    '     
    Set leftWord = rFind.Previous(wdWord, 1)
    Set rightWord = rFind.Next(wdWord, 1)
    Call rightWord.MoveEndWhile(CSet:=" ^t", Count:=wdBackward) '    
    
    leftFlag = False
    rightFlag = False
    
    If rightC Like "[-]" Then
      If Not rFind.Next(wdCharacter, 2) Like "[-]" Then _
        GoTo FIND_NEXT2
      
      Set rightWord = rFind.Next(wdCharacter, 3)
      Do While rightWord.Text Like "[-]"
         Set rightWord = rightWord.Next(wdCharacter, 1)  '     
      Loop
      
      rightFlag = True
      rightWord.Start = rFind.End
      rightWord.End = rightWord.End - 1
    Else
      If Not leftC = Chr(160) And Not rightC = Chr(160) Then
        leftFlag = RangeIsSurname(leftWord)
        If rightC Like "[;,:?!.]" Then
          rightFlag = False
        Else
          rightFlag = RangeIsSurname(rightWord)
        End If
      Else
        If leftC = Chr(160) Then
          Set leftWord = rFind.Previous(wdCharacter, 2)
          Do While leftWord.Text Like "[-]"
            Set leftWord = leftWord.Previous(wdCharacter, 1)   '     
          Loop
          
          leftFlag = leftWord.Start < rFind.Start - 2 And leftWord.Text Like "[-]"
          leftWord.End = rFind.Start - 2
        End If
        If rightC = Chr(160) And rFind.Next(wdCharacter, 2) Like "[-]" And _
            rFind.Next(wdCharacter, 3) Like "[-]" Then
          Set rightWord = rFind.Next(wdCharacter, 4)
          Do While rightWord.Text Like "[-]"
             Set rightWord = rightWord.Next(wdCharacter, 1)  '     
          Loop
          
          rightFlag = True
          rightWord.Start = rFind.End + 1
          rightWord.End = rightWord.End - 1
        End If
      End If
    End If
    
    If Not leftFlag And Not rightFlag Then _
      GoTo FIND_NEXT2
      
    If Not applyFix_ Or (leftFlag And rightFlag) Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION)
      Call IncrementErrors
      GoTo FIND_NEXT2
    End If
    
    newText = rFind.Text
    newText = Left(newText, 2) & Chr(160) & Right(newText, 2) & Chr(160)
    If leftFlag Then  '  ..
      rFind.Start = leftWord.Start
      
      newText = newText & Trim(leftWord.Text)
      rFind.Text = newText
    Else  ' .. 
      rFind.Start = rFind.Start
      rFind.End = rightWord.End
      
      newText = newText & Trim(rightWord.Text)
    End If
    
    If rFind.Fields.Count > 0 Then _
      GoTo FIND_NEXT2
    
    rFind.Text = newText
    Call Increment(doBoth:=True)

FIND_NEXT2:
    Call rFind.Collapse(wdCollapseEnd)
  Loop
End Function

Public Function Rule1017()
  Const RULE_DESCRIPTION$ = "AR1017: "
  
  Dim rFind As Word.Range: Set rFind = editor_.SetupFindWildcard(MASK_QUOTE)
  rFind.Find.Forward = False '   !
  Dim nLevel&: nLevel = 0
  Dim innerQuote As Boolean:  innerQuote = False
  Do While rFind.Find.Execute
    Dim nextC$: nextC = rFind.Next(wdCharacter, 1).Text
    Dim prevC$: prevC = rFind.Previous(wdCharacter, 1).Text
    If nextC = "_" Or prevC = "_" Then _
      GoTo NEXT_FIND  '  
    
    Dim quoteChar$
    If IsQuoteOpen(nextC, prevC, nLevel) Then
      nLevel = nLevel - 1
      quoteChar = IIf(innerQuote, "", "")
      innerQuote = False
      If nextC Like "[]" Then
        Call editor_.MarkError(rFind, RULE_DESCRIPTION & vbNewLine & "  ")
        Call IncrementErrors
      End If
    Else
      nLevel = nLevel + 1
      If innerQuote Then
        Call editor_.MarkError(rFind, RULE_DESCRIPTION & vbNewLine & "  ")
        Call IncrementErrors
      End If
      If nextC Like MASK_QUOTE Then
        innerQuote = True
        quoteChar = ChrW(8220)
      Else
        quoteChar = ""
      End If
    End If

    If nLevel < 0 Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION & vbNewLine & "-   ")
      Call IncrementErrors
      nLevel = 0
    ElseIf nLevel > 3 Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION & vbNewLine & "   > 3")
      Call IncrementErrors
      nLevel = nLevel - 1
    End If

    If rFind.Text <> quoteChar Then
      If Not applyFix_ Then
        Call editor_.MarkError(rFind, RULE_DESCRIPTION & vbNewLine & "  ")
      Else
        rFind.Text = quoteChar
      End If
      Call Increment(applyFix_)
    End If
NEXT_FIND:
  Loop
  
  If nLevel <> 0 Then
    Call editor_.AddGlobalComment(RULE_DESCRIPTION & vbNewLine & "   : " & nLevel)
    Call IncrementErrors
  End If
End Function

Private Function IsQuoteOpen(sNextChar$, sPrevChar$, nLevel&) As Boolean
  If sNextChar Like MASK_ALPHANUM Then
    IsQuoteOpen = True
  ElseIf sNextChar = Chr(13) Then
    IsQuoteOpen = False
  ElseIf sPrevChar = Chr(13) Then
    IsQuoteOpen = True
  ElseIf sPrevChar Like "[().;!\?" & Chr(133) & SYMBOL_DIGIT & SYMBOL_LETTER & SYMBOL_QUOTE & "]" Then
    IsQuoteOpen = False
  Else
    IsQuoteOpen = nLevel > 0
  End If
End Function

Public Function Rule1018()
  Const RULE_DESCRIPTION$ = "AR1018:    "

  Dim aCell As Word.Cell, aRow As Word.Row, aTable As Word.Table
  Dim theWord As Word.Range
  Dim tableRange As Word.Range
  Dim prevWord$
  For Each aTable In editor_.Document.Tables
    Set tableRange = aTable.Range
    For Each aCell In tableRange.Cells
      If aCell.Range.Words.Count <= 1 Then _
        GoTo NEXT_CELL
      
      Set theWord = aCell.Range.Words(aCell.Range.Words.Count - 1)
      If theWord Is Nothing Then _
        GoTo NEXT_CELL
       
      Dim lastword$: lastword = theWord.Text
      If Not lastword Like "*[\!\?;.," & Chr(13) & "]" Then _
        GoTo NEXT_CELL
      
      If lastword = "." Then
        prevWord = theWord.Previous(wdWord, 1).Text
        If prevWord = "" Or prevWord = "" Or prevWord = "" Or prevWord = "" _
          Or prevWord = "" Or prevWord = "" Or prevWord = "" Or prevWord = "" _
          Then GoTo NEXT_CELL
      End If
      
      Call Increment(applyFix_)
      Set theWord = theWord.Characters.Last
      
      If Not applyFix_ Then
        Call editor_.MarkError(aCell.Range, RULE_DESCRIPTION)
        GoTo NEXT_CELL
      End If
      
      theWord.Text = vbNullString
NEXT_CELL:
    Next aCell
  Next aTable
End Function

Public Function Rule1019()
  Const RULE_DESCRIPTION$ = "AR1019:    10 "
  
  Dim tableOfCont As Word.Range
  Dim checkToC As Boolean: checkToC = editor_.Document.TablesOfContents.Count > 0
  If checkToC Then
    Set tableOfCont = editor_.Document.TablesOfContents(1).Range
    Call tableOfCont.MoveEnd(wdParagraph, 1)
  Else
    Set tableOfCont = Nothing
  End If
  
  Dim aPar As Word.Paragraph
  Dim theRange As Word.Range
  For Each aPar In editor_.Document.Paragraphs
    Set theRange = aPar.Range
        
    If theRange.Font.Hidden Then _
      GoTo NEXT_PARAGRAPH
    If theRange.Tables.Count > 0 Then _
      GoTo NEXT_PARAGRAPH
    If checkToC Then _
      If theRange.InRange(tableOfCont) Then _
        GoTo NEXT_PARAGRAPH
    
    If theRange.Words.Count < 120 Then _
      GoTo NEXT_PARAGRAPH
    If theRange.ComputeStatistics(wdStatisticLines) <= AR1019_LINES_PER_PAR Then _
      GoTo NEXT_PARAGRAPH
    
    Call editor_.MarkError(theRange, RULE_DESCRIPTION)
    Call IncrementErrors
NEXT_PARAGRAPH:
  Next aPar
End Function

Public Function Rule1020()
  Const RULE_DESCRIPTION$ = "AR1020:  "
  
  Dim bannedList As Collection: Set bannedList = config_.GetValue("BannedWords")
  Dim rFind As Word.Range
  Dim sWord As Variant
  For Each sWord In bannedList
    Set rFind = editor_.Document.Range
    With rFind.Find
      .MatchWholeWord = True
      .Text = CStr(sWord)
      .Format = True
      .Font.Hidden = False
    End With
    
    Do While rFind.Find.Execute
      Call editor_.MarkError(rFind, RULE_DESCRIPTION)
      Call IncrementErrors
      Call rFind.Collapse(wdCollapseEnd)
    Loop
  Next sWord
End Function

Public Function Rule1021()
  Const RULE_DESCRIPTION$ = "AR1021:  "
  
  Dim theRange As Word.Range: Set theRange = editor_.Document.Paragraphs.Last.Range
  Dim nextRange As Word.Range
  
  Dim level&, newLevel&
  level = 0
  Do While Not theRange Is Nothing
    Set nextRange = theRange.Previous(wdParagraph, 1)
    
    If theRange.Tables.Count > 0 Then _
      GoTo RESET_LEVEL
    If Not theRange.ParagraphFormat.OutlineLevel = wdOutlineLevelBodyText Then _
      GoTo RESET_LEVEL
    If theRange.Font.Hidden = True Then _
      GoTo RESET_LEVEL
    If theRange.Tables.Count > 0 Then _
      GoTo RESET_LEVEL
    
    newLevel = theRange.ListFormat.ListLevelNumber
    If theRange.ListFormat.ListValue = 0 And theRange.ListFormat.ListString = vbNullString Then _
      newLevel = 0
    
    If level > newLevel And Not theRange.ParagraphFormat.KeepWithNext Then
      If Not applyFix_ Then
        Call editor_.MarkError(theRange, RULE_DESCRIPTION)
      Else
        theRange.ParagraphFormat.KeepWithNext = True
      End If
      Call Increment(applyFix_)
    End If
    
    level = newLevel
    GoTo SKIP_RESET
RESET_LEVEL:
    level = 0
SKIP_RESET:
    Set theRange = nextRange
  Loop
End Function

Public Function Rule1022()
  Const RULE_DESCRIPTION$ = "AR1022:   "
  
  Dim numLists&:  numLists = editor_.Document.ListParagraphs.Count
  Dim lastChar$
  Dim theRange As Word.Range
  Dim nextRange As Word.Range
  Dim nItem&
  For nItem = 1 To numLists Step 1
  '   
    Set theRange = editor_.Document.ListParagraphs(nItem).Range
    If Not theRange.ParagraphFormat.OutlineLevel = wdOutlineLevelBodyText Then _
      GoTo NEXT_LIST_ITEM
    If theRange.Tables.Count > 0 Then _
      GoTo NEXT_LIST_ITEM
    
    Set nextRange = theRange.Next
    Call theRange.MoveEndWhile(Chr(13) & Chr(7) & Chr(12) & Chr(14), wdBackward)
    If theRange.Start = theRange.End Then _
      GoTo NEXT_LIST_ITEM
    lastChar = theRange.Characters.Last
    
    If IsList(nextRange) Then
      If lastChar Like "[.?!,:;]" Then _
        GoTo NEXT_LIST_ITEM
      Call IncrementErrors
      Call editor_.MarkError(theRange.Words.Last, RULE_DESCRIPTION & vbNewLine & "     ")
      GoTo NEXT_LIST_ITEM
    End If
    
    If lastChar Like "[.?!]" Then _
      GoTo NEXT_LIST_ITEM
    If lastChar Like "[;,]" And Left(nextRange.Text, 1) Like "[a-z-]" Then _
      GoTo NEXT_LIST_ITEM
    
    If applyFix_ Then
      If lastChar Like "[;,]" Then
        theRange.Characters.Last = "."
      Else
        Call theRange.InsertAfter(".")
      End If
    Else
      Call editor_.MarkError(theRange, RULE_DESCRIPTION)
    End If
    Call Increment(applyFix_)
NEXT_LIST_ITEM:
  Next nItem
End Function

Public Function Rule1023()
  Const RULE_DESCRIPTION$ = "AR1023:    "
  
  Dim result&: result = 0
  result = result + editor_.ReplaceCascade("<[].^0032" & MASK_DIGIT, "^0032", "^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[].^0032" & MASK_DIGIT, "^0032", "^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[].^0032" & MASK_DIGIT, "^0032", "^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[].^0032" & MASK_DIGIT, "^0032", "^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[]." & MASK_DIGIT, ".", ".^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[]." & MASK_DIGIT, ".", ".^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[]." & MASK_DIGIT, ".", ".^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[]." & MASK_DIGIT, ".", ".^0160", applyFix_, RULE_DESCRIPTION)
  Call Increment(applyFix_, result)
End Function

Public Function Rule1024()
  Const RULE_DESCRIPTION$ = "AR1024:  "
' TODO: refactor!
  Dim brStack As New Collection
  Dim stackItem As BracketItem
  Dim openBr As Boolean
  Dim brType As BracketType
  
  Dim i&, totalstack&

  Dim nStartPara&:  nStartPara = NEW_PARAGRAPH
  Dim firstPar As Word.Range
  Dim firstWord$
  
  Dim rFind As Word.Range
  Set rFind = editor_.SetupFindWildcard("[\{\}\(\)\[\]]")
  Do While rFind.Find.Execute
    brType = GetBracketType(rFind.Text)  'get bracket type
    
    '  -   (   )
    Set firstPar = rFind.Paragraphs.First.Range
    If Not firstPar.Start = nStartPara Then
      If firstPar.Words.Count < 2 Or brType <> BT_PAP Then _
        GoTo CONTINUE
      If firstPar.Words(2).Start <> rFind.Start Then _
        GoTo CONTINUE
      If Not rFind.Next(wdCharacter, 1) Like "[ " & Chr(9) & "]" Then _
        GoTo CONTINUE
      
      firstWord = firstPar.Words(1)
      If firstWord Like MASK_ROMAN_DIGIT & "*" Or firstWord Like MASK_LETTER Or IsNumeric(firstWord) Then _
        GoTo NEXT_BRACKET

CONTINUE:
      If nStartPara = NEW_PARAGRAPH Then _
        GoTo SKIP_STACK_CLEAR
      For i = 1 To brStack.Count
        Call editor_.MarkError(brStack.Item(i).brRange, RULE_DESCRIPTION & vbNewLine & "  ")
        Call IncrementErrors
      Next i
      Set brStack = New Collection
      nStartPara = NEW_PARAGRAPH
    End If
    
SKIP_STACK_CLEAR:
    openBr = rFind.Text Like "[\(\{\[]"   'push
    If openBr Then
      Set stackItem = New BracketItem
      Set stackItem.brRange = editor_.Document.Range(rFind.Start, rFind.End)
      stackItem.brType = brType
      stackItem.brMark = False
      Call brStack.Add(stackItem)
      
      If nStartPara = -1 Then _
        nStartPara = rFind.Paragraphs.First.Range.Start
      
      GoTo NEXT_BRACKET
    End If
    
    ' ,   
    '  -     (  )
    
    totalstack = brStack.Count
    If totalstack <> 0 Then
      Do While Not brStack.Item(totalstack).brType = brType
         brStack.Item(totalstack).brMark = True
         totalstack = totalstack - 1
         If totalstack = 0 Then _
          Exit Do
      Loop
    End If

    '      ( )
    If totalstack = 0 Then
      Call editor_.MarkError(rFind, RULE_DESCRIPTION & vbNewLine & "  ")
      Call IncrementErrors
      GoTo NEXT_BRACKET
    End If
    
    '   -   
    If brStack.Item(totalstack).brMark Then
      Call editor_.MarkError(brStack.Item(totalstack).brRange, RULE_DESCRIPTION & vbNewLine & " ")
      Call IncrementErrors
    End If
    
    ' ,    (     )
    Call brStack.Remove(totalstack)
NEXT_BRACKET:
  Loop
  
  '     
  For i = 1 To brStack.Count
    Call editor_.MarkError(brStack.Item(i).brRange, RULE_DESCRIPTION & vbNewLine & "  ")
    Call IncrementErrors
  Next i
End Function

Public Function Rule1025()
  Const RULE_DESCRIPTION$ = "AR1025:  "

  Dim theRange As Word.Range, prevRange As Word.Range
  Dim firstPar As Word.Range: Set firstPar = editor_.Document.Paragraphs.First.Range
  Dim theLvl&, prevLvl&
  Dim para As Word.Paragraph
  For Each para In editor_.Document.ListParagraphs
    Set theRange = para.Range
    theLvl = theRange.ListFormat.ListLevelNumber
    If InStr(theRange, Chr(7)) Then _
      GoTo NEXT_PARA
    
    Set prevRange = theRange.Previous(wdParagraph)
    On Error Resume Next
    If Not theRange.ListFormat.ListString Like "*#*" Or IsRangeHeader(theRange) Then _
      GoTo NEXT_PARA
    If prevRange Is Nothing Or IsRangeHeader(prevRange) Or _
      prevRange.ListFormat.List Is Nothing Or _
      theLvl > prevRange.ListFormat.ListLevelNumber Then

      If theRange.ListFormat.ListValue <> 1 Then
        Call theRange.Collapse(wdCollapseStart)
        Call editor_.MarkError(theRange, RULE_DESCRIPTION & vbNewLine & "  ")
        Call IncrementErrors
      End If
    End If
        
NEXT_PARA:
  Next para
End Function

Public Function Rule1026()
  Const RULE_DESCRIPTION$ = "AR1026:  "
  
  Dim result&: result = 0
  result = result + editor_.ReplaceCascade("<[]." & "^0160" & ".", "." & "^0160" & ".", "" & "^0160" & "", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[]." & "^0032" & ".", "." & "^0032" & ".", "" & "^0160" & "", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[]..", "..", "" & "^0160" & "", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[].[].", ".", ".^0160", applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<[].^0032[].", "^0032", "^0160", applyFix_, RULE_DESCRIPTION)
  Call Increment(applyFix_, result)
End Function

Public Function Rule1027()
  Const RULE_DESCRIPTION$ = "AR1027:  "
  
  Dim tmpRef As Word.Field
  Dim codeString$
  Dim startPos&
  For Each tmpRef In editor_.Document.Fields
  
    If Not (tmpRef.Type = wdFieldRef And (tmpRef.result Like "[]*" Or tmpRef.result Like "[]*")) Then _
        GoTo SKIP_LOWERING
        
    If InStr(tmpRef.Code.Text, "\* Lower") = 0 Then
      Call IncrementErrors
      
      If Not applyFix_ Then
        Call editor_.MarkError(tmpRef.result, RULE_DESCRIPTION)
        GoTo SKIP_LOWERING
      End If
      
      codeString = tmpRef.Code.Text
      startPos = InStr(codeString, "_Ref")
      Do While Mid(codeString, startPos, 1) <> " "
        startPos = startPos + 1
      Loop
      codeString = VBA.Left(codeString, startPos) & "\* Lower " & VBA.Mid(codeString, startPos + 1)
      tmpRef.Code.Text = codeString
      tmpRef.Update
    End If
SKIP_LOWERING:
  Next tmpRef
End Function

Public Function Rule1028()
  Const RULE_DESCRIPTION$ = "AR1028:   "
  
  Dim iFoot As Word.Footnote
  Dim iEnd As Word.Endnote
  Dim preRange As Word.Range
  For Each iFoot In editor_.Document.Footnotes
    Set preRange = iFoot.Reference.Previous(wdCharacter)
    If Asc(preRange) = 32 Or Asc(preRange) = 160 Then
      Call Increment(applyFix_)
      If applyFix_ Then
        Call preRange.MoveStartWhile(" " & Chr(160), wdBackward)
        Call preRange.Delete
      Else
        Call editor_.MarkError(preRange, RULE_DESCRIPTION)
      End If
    End If
  Next iFoot
  
  For Each iEnd In editor_.Document.Endnotes
    Set preRange = iEnd.Reference.Previous(wdCharacter)
    If Asc(preRange) = 32 Or Asc(preRange) = 160 Then
      Call Increment(applyFix_)
      If applyFix_ Then
        Call preRange.MoveStartWhile(" " & Chr(160), wdBackward)
        Call preRange.Delete
      Else
        Call editor_.MarkError(preRange, RULE_DESCRIPTION)
      End If
    End If
  Next iEnd
End Function

Public Function Rule1029()
  Const RULE_DESCRIPTION$ = "AR1029:  /"
  
  Dim result&: result = 0
  Dim badTail$:   badTail = "[" & "^0160" & "^0032" & "]"
  Dim repWith$:   repWith = "^0160" & "()"
  
  result = result + editor_.ReplaceCascade("</" & badTail & ">", "/" & badTail, repWith, applyFix_, RULE_DESCRIPTION, bWildcard:=True)
  result = result + editor_.ReplaceCascade("<^0032/" & badTail & ">", " /" & badTail, repWith, applyFix_, RULE_DESCRIPTION, bWildcard:=True)
  result = result + editor_.ReplaceCascade("</>", "/", repWith, applyFix_, RULE_DESCRIPTION)
  result = result + editor_.ReplaceCascade("<^0032/>", " /", repWith, applyFix_, RULE_DESCRIPTION)
  
  Call Increment(applyFix_, result)
End Function

Public Function Rule1098()
  Const RULE_DESCRIPTION$ = "AR1098:   Word"
  
  Dim nSpelling&: nSpelling = editor_.Document.Range.SpellingErrors.Count
  Dim nGrammar&: nGrammar = editor_.Document.Range.GrammaticalErrors.Count
  If nSpelling + nGrammar > 0 Then _
    Call editor_.AddGlobalComment(Fmt(TEXT_SPELLING_STATS, RULE_DESCRIPTION, nSpelling, nGrammar))
  
  Call IncrementErrors(nSpelling + nGrammar)
End Function

Public Function Rule1099()
  Const RULE_DESCRIPTION$ = "AR1099:  "
  
  Dim tmp As Boolean: tmp = editor_.Document.TrackRevisions
  editor_.Document.TrackRevisions = False
  
  Dim aField As Word.Field
  For Each aField In editor_.Document.Fields
    If Not aField.Update Then
      If Not aField.result.Font.Hidden Then
        Call editor_.MarkError(aField.result, RULE_DESCRIPTION)
        Call IncrementErrors
      End If
    End If
  Next aField
  
  editor_.Document.TrackRevisions = tmp
End Function

' ===========
Private Function Increment(doBoth As Boolean, Optional nInc& = 1)
  errors_ = errors_ + nInc
  If doBoth Then _
    fixes_ = fixes_ + nInc
End Function

Private Function IncrementFixes(Optional nInc& = 1)
  fixes_ = fixes_ + nInc
End Function

Private Function IncrementErrors(Optional nInc& = 1)
  errors_ = errors_ + nInc
End Function

Private Function RangeIsSurname(subject As Word.Range) As Boolean
  RangeIsSurname = False
  
  Dim sStr$:  sStr = VBA.Trim(subject.Text)
  If sStr Like "*[^0160.]" Then _
    Exit Function
  If Not sStr Like "[-]*[-]" Then _
    Exit Function

  RangeIsSurname = True
End Function

Private Function IsRangeHeader(theRange As Word.Range) As Boolean
  If theRange Is Nothing Then
    IsRangeHeader = False
    Exit Function
  End If
  IsRangeHeader = theRange.ParagraphFormat.OutlineLevel <> wdOutlineLevelBodyText _
      Or theRange.Style Like "*[]*"
End Function

Private Function IsList(aRange As Word.Range) As Boolean
'       
  If aRange Is Nothing Then
    IsList = True
    Exit Function
  End If
  IsList = aRange.ListFormat.ListValue = 0 And aRange.ListFormat.ListString = vbNullString And aRange.ListFormat.ListLevelNumber = 0
  IsList = Not IsList
End Function

Private Function ReplaceAliasDig(targetText$, aText$) As Long
'        
  Dim replText$:   replText = Chr(160) & targetText
  Dim result&
  result = editor_.ReplaceCascade("([0-9]{2;4})" & targetText, targetText, replText, applyFix_, aText)
  result = result + editor_.ReplaceCascade("([0-9]{2;4}) " & targetText, " ", Chr(160), applyFix_, aText)
  ReplaceAliasDig = result
End Function

Private Function RunRulesInternal(iExclude As Scripting.Dictionary)
  If Not iExclude.Exists("AR1001") Then Call Rule1001
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1002") Then Call Rule1002
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1003") Then Call Rule1003
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1004") Then Call Rule1004
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1005") Then Call Rule1005
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1006") Then Call Rule1006
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1007") Then Call Rule1007
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1008") Then Call Rule1008
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1009") Then Call Rule1009
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1010") Then Call Rule1010
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1011") Then Call Rule1011
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1012") Then Call Rule1012
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1013") Then Call Rule1013
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1014") Then Call Rule1014
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1016") Then Call Rule1016
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1017") Then Call Rule1017
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1018") Then Call Rule1018
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1019") Then Call Rule1019
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1020") Then Call Rule1020
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1021") Then Call Rule1021
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1022") Then Call Rule1022
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1023") Then Call Rule1023
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1024") Then Call Rule1024
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1025") Then Call Rule1025
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1026") Then Call Rule1026
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1027") Then Call Rule1027
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1028") Then Call Rule1028
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1029") Then Call Rule1029
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1098") Then Call Rule1098
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
  If Not iExclude.Exists("AR1099") Then Call Rule1099
  If CSE_ProgressBar.Visible Then Call CSE_ProgressBar.IncrementA
End Function
