VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_RulesProcessor"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' TODO: test all missing rules

Private processor_ As RulesProcessor
Private doc_ As Word.Document
Private ruleID_ As String

Public Function Setup()
  Set doc_ = ThisDocument
  Set processor_ = New RulesProcessor
  Call processor_.Init(doc_, EmptyEditConfig)
End Function

Public Function Teardown()
  Call doc_.Range.Delete
End Function

Private Function TestText(sInitial$, sExpected$, Optional sMsg$ = vbNullString)
  Dim nStart&: nStart = doc_.Range.End
  Call doc_.Paragraphs.Add.Range.InsertAfter(sInitial)
  
  Call processor_.ResetCounters
  Call processor_.RunRule(ruleID_, bApplyFix:=True)
  
  Dim iLastRange As Word.Range: Set iLastRange = doc_.Range.Duplicate
  iLastRange.Start = nStart
  iLastRange.End = iLastRange.End - 1
  Dim sTest$: sTest = iLastRange.Text
  Call doc_.Range.Delete
  
  Call Dev_ExpectEQ(sExpected, sTest, sMsg)
End Function

Private Function TestComment(sInitial$, sAnchor$, Optional sComment$ = vbNullString, Optional sClause$ = vbNullString)
  If sClause <> "" Then _
    Call Dev_NewClause(sClause)
  
  Call doc_.Paragraphs.Add.Range.InsertAfter(sInitial)
  
  Call processor_.RunRule(ruleID_, bApplyFix:=True)
  Call processor_.FinalizeProcessing
  Call Dev_AssertEQ(1, doc_.Comments.Count, "Comments count")
  
  Dim iComment As Word.Comment: Set iComment = doc_.Comments(1)
  
  Dim sText$: sText = iComment.Range.Text
  Call Dev_ExpectEQ(sAnchor, iComment.Scope, "Anchor text")
  If sComment <> "" Then
    Call Dev_ExpectLike(sText, "*" & sComment, "Comment text")
  End If
  
  Call doc_.Range.Delete
End Function

Public Function t_HiddenText()
  On Error GoTo PROPAGATE_ERROR
  ruleID_ = "AR1002"
  
  Dim sText$: sText = "Test1  Test2"
  Dim nStart&: nStart = doc_.Range.End
  Call doc_.Paragraphs.Add.Range.InsertAfter(sText)
  Dim iLastRange As Word.Range: Set iLastRange = doc_.Range.Duplicate
  iLastRange.Start = nStart
  iLastRange.End = iLastRange.End - 1
  iLastRange.Font.Hidden = True
  
  Call processor_.RunRule(ruleID_, bApplyFix:=True)
  
  Set iLastRange = doc_.Range.Duplicate
  iLastRange.Start = nStart
  iLastRange.End = iLastRange.End - 1
  Dim sTest$: sTest = iLastRange.Text
  
  Call Dev_ExpectEQ(sText, sTest, "Do not apply rule to hidden text")
  
  iLastRange.Font.Hidden = False
  Call processor_.RunRule(ruleID_, bApplyFix:=True)
  
  Set iLastRange = doc_.Range.Duplicate
  iLastRange.Start = nStart
  iLastRange.End = iLastRange.End - 1
  sTest = iLastRange.Text
  Call Dev_ExpectNE(sText, sTest, "Apply valid rule to not hidden text")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ParagraphWhitespace()
  On Error GoTo PROPAGATE_ERROR
  ruleID_ = "AR1001"
  
  Call Dev_NewCase("Space in front")
  Call TestText(" Test", "Test", "Single space")
  Call TestText("   Test", "Test", "Multiple spaces")
  Call TestText(Chr(160) & "Test", "Test", "Unbreakable space")
  Call TestText(Chr(160) & " " & Chr(160) & "Test", "Test", "Mixed spaces")
  Call TestText(vbTab & "Test", vbTab & "Test", "Ignore tabulator")
  Call TestText(" " & vbTab & "Test", vbTab & "Test", "Mixed tabulator")
  
  Call Dev_NewCase("Space in back")
  Call TestText("Test ", "Test", "Single space")
  Call TestText("Test   ", "Test", "Multiple spaces")
  Call TestText("Test" & Chr(160), "Test", "Unbreakable space")
  Call TestText("Test" & Chr(160) & " " & Chr(160), "Test", "Mixed spaces")
  Call TestText("Test" & vbTab, "Test", "Tabulator")
  Call TestText("Test" & vbTab & vbTab, "Test", "Multiple tabulator")
  Call TestText("Test" & " " & vbTab, "Test", "Mixed tabulator")
  
  Call Dev_NewCase("Combo spaces")
  Call TestText("  Test  " & vbTab, "Test")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_DoubleSpace()
  On Error GoTo PROPAGATE_ERROR
  ruleID_ = "AR1002"
  
  Call TestText("Test1  test2", "Test1 test2", "Double space")
  Call TestText("Test1 " & Chr(160) & "test2", "Test1 test2", "Double unbreakable space")
  Call TestText("Test1   test2", "Test1 test2", "Triple space")
  Call TestText("Test1" & vbTab & vbTab & "test2", "Test1" & vbTab & vbTab & "test2", "Double tab")
  
  Call Dev_NewCase("Quadruple space")
  Call TestText("Test1    test2", "Test1 test2", "Replace text")
  Call Dev_ExpectEQ(1, processor_.errors_, "Error count")
  Call Dev_ExpectEQ(1, processor_.errors_, "Fixes count")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_DoubleEndline()
  On Error GoTo PROPAGATE_ERROR
  ruleID_ = "AR1003"
  
  Call TestText(vbNewLine & "Test", "Test", "Double endline")
  Call TestText("Test" & vbNewLine, "Test", "End of document")
  
  Call Dev_NewCase("Triple endline")
  Call TestText(vbNewLine & vbNewLine & "Test", "Test", "Replace text")
  Call Dev_ExpectEQ(1, processor_.errors_, "Error count")
  Call Dev_ExpectEQ(1, processor_.errors_, "Fixes count")
  
  Call Dev_NewCase("Quadruple endline")
  Call TestText(vbNewLine & vbNewLine & vbNewLine & "Test", "Test", "Replace text")
  Call Dev_ExpectEQ(1, processor_.errors_, "Error count")
  Call Dev_ExpectEQ(1, processor_.errors_, "Fixes count")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_PlusMinus()
  On Error GoTo PROPAGATE_ERROR
  ruleID_ = "AR1004"
  
  Call TestText("1 +/- 1", "1  1", "Spaces")
  Call TestText("1+/-1", "11", "No spaces")
  Call TestText("1 + / - 1", "1  1", "Inner spaces")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Quotes()
  On Error GoTo PROPAGATE_ERROR
  ruleID_ = "AR1017"
  
  Call Dev_NewCase("Replace incorrect brackets")
  Call TestText("Test", "Test")
  Call TestText("""Test""", "Test")
  Call TestText("Test", "Test")
  Call TestText("Test", "Test")
  Call TestText("""""", "")
  Call TestText("", "")
  
  Call Dev_NewCase("Double brackets")
  Call TestText("  һ", "  ғ")
  Call TestText("""  ҫ", "  ғ")
  Call TestText("  һ INC", "  һ INC")
  
  Call Dev_NewCase("Invalid structure")
  Call TestComment("Test", "", sClause:="Missing close bracket")
  Call TestComment("Test 12", "", sClause:="Missing close bracket")
  Call TestComment("Test", "", sClause:="Missing open bracket")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

