Attribute VB_Name = "z_UIMessages"
' Messaging module
Option Private Module
Option Explicit

Public Enum MsgCode
  EM_HYPERLINK_CREATION_FAIL
  EM_HYPERLINKS_MISSING
  EM_DOCUMENT_NOT_SAVED
  EM_CORE_UPDATE_FAILED
  EM_SELECTION_EMPTY
  EM_MISSING_FILE
  
  IM_REMOVE_HYPERLINKS_OK
  IM_HYPERLINKS_FIX_OK
  IM_REMOVE_BOOKMARKS_OK
  IM_REMOVE_IMAGES_OK
  IM_SHAPES_TO_TEXT_OK
  IM_CONVERT_CODEPAGE_OK
  IM_UNIQUE_BOOKMARKS_OK
  IM_CONCEPT_UPDATE_OK
  IM_BOOKMARK_IGNORED
  IM_RULES_COMPLETE
  IM_REMOVE_ACCENTS_OK
  IM_COMMENT_EXPORT_OK
End Enum

Private g_UI As API_UserInteraction

Public Function UserInteraction() As API_UserInteraction
  If g_UI Is Nothing Then _
    Set g_UI = New API_UserInteraction
  Set UserInteraction = g_UI
End Function

Public Function SetUserInteraction(newUI As API_UserInteraction)
  Set g_UI = newUI
End Function

Public Function UIShowMessage(theCode As MsgCode, ParamArray params() As Variant)
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Select Case theCode
  Case EM_HYPERLINK_CREATION_FAIL:  Call MsgBox("    .   ", vbExclamation)
  Case EM_HYPERLINKS_MISSING:       Call MsgBox("   ", vbInformation)
  Case EM_DOCUMENT_NOT_SAVED:       Call MsgBox("   " & vbNewLine & "    ", vbCritical)
  Case EM_SELECTION_EMPTY:          Call MsgBox(" .   ", vbExclamation)
  Case EM_MISSING_FILE:             Call MsgBox(Fmt("   : {1}", unwrapped), vbExclamation)
  Case EM_CORE_UPDATE_FAILED:       Call MsgBox("      ." & vbNewLine _
                                                  & " VPN,   (\\fs1.concept.ru)" & vbNewLine _
                                                  & "  Excel   dll", vbExclamation)

  Case IM_REMOVE_HYPERLINKS_OK:     Call MsgBox(Fmt(" : {1}", unwrapped), vbInformation)
  Case IM_REMOVE_BOOKMARKS_OK:      Call MsgBox(Fmt(" : {1}", unwrapped), vbInformation)
  Case IM_REMOVE_IMAGES_OK:         Call MsgBox(Fmt(" : {1}", unwrapped), vbInformation)
  Case IM_SHAPES_TO_TEXT_OK:        Call MsgBox("  ", vbInformation)
  Case IM_CONVERT_CODEPAGE_OK:      Call MsgBox(" ", vbInformation)
  Case IM_UNIQUE_BOOKMARKS_OK:      Call MsgBox("  ", vbInformation)
  Case IM_REMOVE_ACCENTS_OK:        Call MsgBox(Fmt(" : {1}", unwrapped), vbInformation)
  Case IM_CONCEPT_UPDATE_OK:        Call MsgBox("    ", vbInformation)
  Case IM_COMMENT_EXPORT_OK:        Call MsgBox(Fmt(" : {1}", unwrapped), vbInformation)
  Case IM_BOOKMARK_IGNORED:
    Call MsgBox(Fmt("    " & vbNewLine & _
                "      {1}", unwrapped), vbInformation)
  Case IM_HYPERLINKS_FIX_OK:
    Call MsgBox(Fmt(" : {1}" & vbNewLine & _
                    " : {2}" & vbNewLine & _
                    ": {3}", unwrapped), vbInformation)
  Case IM_RULES_COMPLETE
    Call MsgBox(Fmt(" : {1} ms" & vbNewLine & vbNewLine & _
                    " : {2}" & vbNewLine & _
                    " : {3}", unwrapped), vbInformation)

  Case Else:                        Call MsgBox("  ", vbCritical)
  End Select
End Function

Public Function UIAskQuestion(theCode As MsgCode, ParamArray params() As Variant) As Boolean
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Dim answer&: answer = vbNo
  Select Case theCode
'  Case QM_CODE_DELETE_CONFIRM
'    answer = MsgBox("Are you sure you want to delete ALL macros from target file?", vbYesNo + vbQuestion)
    
  Case Else
    Call MsgBox("  ", vbCritical)
  End Select
  UIAskQuestion = answer = vbYes
End Function
